function [A,B,C]=ftpara03(X,r1,r2,r3,W,startmode,convlim,Ai,Bi,Ci);
%function [A B C]=ftpara03(X,r1,r2,r3,W,startmode,convlim,Ai,Bi,Ci);
%
%This PARAFAC/CANDECOMP program calculates the W factors in
%the problem X approx. = sum (A(w)*B(w)*C(w)) for w=1,2,...,W;
%Solutions are found as the least squares solutions to the 
%equation: min( sum(w) ||X(w) - AD(w)B'|| ), w=1,2,..,W;
%
%This particular procedure (ftpara03) lay constaints on B and C:
%B and C are forced to be normalized, thereby accumulating most of the 
%explanatory variation in the A-factor which is left unconstrained.
%This gives the A-factors more meaning as score-vectors. 
%This procedure is equipped with an accelerator.
%
% X        : Supermatrix (r1 x r2r3) from cube (r1 x r2 x r3).
% r1,r2,r3 : Number of observations along each way.
% W        : Number of factors.
% startmode: 1=Initiate A, B and C by TUCKER1/UPCA solutions.
%            2=Initiate A, B and C with Ai, Bi and Ci.
%            3=Random vectors (propably fastest).
% convlim  : Max. difference between two succesive sq. sums of misfits.
%            Default set to 1.0e-2 
% Ai,Bi,Ci : Initial guesses on A, B and C from the user.
% A,B,C	   : The found factors in columns. 
%            The dimensions will be A(r1 x W) B(r2 x W) C(r3 x W).
% Author   : Claus A. Andersson, May 1995 
% Copyright: Food Technology,
%            Royal Veterinary & Agricultural University
%            Copenhagen, Denmark
% E-mail   : claus.andersson@pop.foodsci.kvl.dk

%Initialize A, B and C
if startmode==1, 
  disp('Initializing A, B and C with TUCKER1/UPCA solutions');
  disp('Calling FTUPCA01.M');
  [A B C]=ftupca01(X,r1,r2,r3,W,W,W);  
end;
if startmode==2,
  disp('Initializing A, B and C as defined by user.');
  A=Ai;
  B=Bi;
  C=Ci;
  clear Ai Bi Ci 
end; 
if startmode==3,
  disp('Initializing A, B and C by random vectors.');
  A=rand(r1,W)+1;
  B=rand(r2,W)+1;
  C=rand(r3,W)+1;
end;

%Calculates the misfit of the factors
diff=sum(sum((X(:,1:r2)-A*diag(C(1,:))*B').^2));
for t=2:r3,
  diff=diff+sum(sum((X(:,(t-1)*r2+1:t*r2)-A*diag(C(t,:))*B').^2));
end;

%Iteration starts
if exist('convlim')==0,
  convlim=1e-2;
end;
if convlim<eps,
  convlim=1e-2;
end;
tmpC=zeros(r3,W);
it=0;
acc=0;
olddiff=diff;
converged=0;
while converged==0,
  it=it+1;

  %Updating A
  Dt=diag(C(1,:));
  tmp1=X(1:r1,1:r2)*B*Dt;
  tmp2=Dt*B'*B*Dt;
  for t=2:r3,
    Dt=diag(C(t,:));
    tmp1=tmp1+X(1:r1,(t-1)*r2+1:t*r2)*B*Dt;
    tmp2=tmp2+Dt*B'*B*Dt;
  end;
  A=tmp1/tmp2;

  %Updating B
  Dt=diag(C(1,:));
  tmp1=X(1:r1,1:r2)'*A*Dt;
  tmp2=Dt*A'*A*Dt;
  for t=2:r3,
    Dt=diag(C(t,:));
    tmp1=tmp1+X(1:r1,(t-1)*r2+1:t*r2)'*A*Dt;
    tmp2=tmp2+Dt*A'*A*Dt;
  end;
  B=tmp1/tmp2;
  for t=1:W,
    B(:,t)=B(:,t)/norm(B(:,t));
  end;

  %Updating C
  for t=1:r3,
    tmpC(t,1:W)=diag(A'*X(1:r1,(t-1)*r2+1:t*r2)*B)';
  end;
  C=tmpC/((A'*A).*(B'*B));
  for t=1:W,
    C(:,t)=C(:,t)/norm(C(:,t));
  end;

  %Calculates the misfit of the factors
  diff=sum(sum((X(:,1:r2)-A*diag(C(1,:))*B').^2));
  for t=2:r3,
    diff=diff+sum(sum((X(:,(t-1)*r2+1:t*r2)-A*diag(C(t,:))*B').^2));
  end;
  fprintf('The sq. sum of misfit is %12.8f after %g iterations. \n',diff,it);
 
  %Accelerator
  if acc==0 & olddiff/diff<1.3,
     acc=1;
     it0=it;
     A1=A;
     B1=B;
  end;
  if it==it0+1,
    Bb=B;
    Cb=C;
    acc=0; 
    A=A1+(A-A1)*it^(1/W);
    B=B1+(B-B1)*it^(1/W);
    for t=1:W,
      B(:,t)=B(:,t)/norm(B(:,t));
    end;
    
    %Updating C
    for t=1:r3,
      tmpC(t,1:W)=diag(A'*X(1:r1,(t-1)*r2+1:t*r2)*B)';
    end;
    C=tmpC/((A'*A).*(B'*B));
    for t=1:W,
      C(:,t)=C(:,t)/norm(C(:,t));
    end;

    %Calculates the misfit of the factors
    ndiff=sum(sum((X(:,1:r2)-A*diag(C(1,:))*B').^2));
    for t=2:r3,
      ndiff=ndiff+sum(sum((X(:,(t-1)*r2+1:t*r2)-A*diag(C(t,:))*B').^2));
    end;

    if ndiff>diff,
      B=Bb;
      for t=1:W,
        B(:,t)=B(:,t)/norm(B(:,t));
      end;
      C=Cb; 
      for t=1:W,
        C(:,t)=C(:,t)/norm(C(:,t));
      end;
      fprintf('Failed to accelerate.\n');
    else
      fprintf('Succeded to accelerate.\n');
    end;
  end;

  %Shows indications of degenerate solutions
  if rem(it,10)==0,
    disp('Triple cosine ');
    dAA=(diag(diag(A'*A))^.5);
    dBB=(diag(diag(B'*B))^.5);
    dCC=(diag(diag(C'*C))^.5);
    phi1=dAA\A'*A/dAA;
    phi2=dBB\B'*B/dBB;
    phi3=dCC\C'*C/dCC;
    disp(phi1.*phi2.*phi3)
    clear dAA dBB dCC
  end;

  %Checks if a minimum has been found
  if abs(olddiff-diff)<convlim, converged=1; end,
  olddiff=diff;

end;

clear olddiff diff tmpC tmp1 tmp2 tmp converged convlim it Bb Cb t
clear ndiff acc A1 B1 it0 Dt

disp('FTPARA03: Execution ended.')